﻿using igGrid_EditingEvents.Models;
using Infragistics.Web.Mvc;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using igGrId_EditingEvents.Models;

namespace igGrid_EditingEvents.Controllers
{
    public class HomeController : Controller
    {
        ProductList pl = new ProductList();
        ProductGroupList pgl = new ProductGroupList();

        // GET: Home
        public ActionResult Index()
        {
            GridModel model = CreateProductsGrid();
            return View(model);
        }

        public JsonResult GetData_Generic(string path, string layout)
        {
            GridModel model = CreateProductsGrid();

            model.DataSource = pl.GetProductList();

            if (layout == null)
            {
                return model.GetData();
            }

            return model.GetData(path, layout);
        }

        [ComboDataSourceAction()]
        public ActionResult GetDistict_Generic(string path, string filter)
        {
            IList<ComboDataViewModel> results = new List<ComboDataViewModel>();

            foreach (var group in pgl.GetProductGroupList())
            {
                ComboDataViewModel dataViewModel = new ComboDataViewModel(group.ProductGroupId, group.ProductGroupCode);
                results.Add(dataViewModel);
            }

            // ReSharper disable once Mvc.ViewNotResolved
            return View(results.AsQueryable());
        }

        private GridModel CreateProductsGrid()
        {
            GridModel gridModel = new GridModel();

            gridModel.DataSourceUrl = Url.Action("GetData_Generic", "Home");
            gridModel.ID = "grid1";
            gridModel.AutoGenerateLayouts = false;
            gridModel.AutoGenerateColumns = false;
            gridModel.AutoCommit = false;
            gridModel.Width = "800px";
            gridModel.Height = "400px";
            gridModel.PrimaryKey = "Id";

            CustomGridColumn col1 = new CustomGridColumn("Id", "Id", "number", "auto") { ColumnGroup = "Group1" };
            gridModel.Columns.Add(col1.Column);

            CustomGridColumn col2 = new CustomGridColumn("Name", "Name", "string", "auto") { ColumnGroup = "Group1" };
            gridModel.Columns.Add(col2.Column);

            CustomGridColumn col3 = new CustomGridColumn("Product Number", "ProductNumber", "number", "auto") { ColumnGroup = "Group2" };
            gridModel.Columns.Add(col3.Column);

            CustomGridColumn col4 = new CustomGridColumn("Product Group Code", "ProductGroupId", "number", "auto") { ColumnGroup = "Group2"};
            gridModel.Columns.Add(col4.Column);

            IList<CustomGridColumn> customColumns = new List<CustomGridColumn>();
            customColumns.Add(col1);
            customColumns.Add(col2);
            customColumns.Add(col3);
            customColumns.Add(col4);

            GridUpdating updating = new GridUpdating();
            updating.EditMode = GridEditMode.Dialog;
            updating.EnableAddRow = true;
            gridModel.Features.Add(updating);

            ColumnUpdatingSetting updatingSetting = new ColumnUpdatingSetting();
            updatingSetting.ColumnKey = "ProductGroupId";
            updatingSetting.EditorType = ColumnEditorType.Combo;
            updatingSetting.ComboEditorOptions.FilteringType = ComboFilteringType.Remote;
            updatingSetting.ComboEditorOptions.FilterExprUrlKey = "filter";
            updatingSetting.EditorProvider = "new EditorProviderRemoteCombo()";
            //updatingSetting.ComboEditorOptions.LoadOnDemandSettings = new ComboLoadOnDemandSettings() { Enabled = true, PageSize = 25 };
            updatingSetting.ComboEditorOptions.TextKey = "Value";
            updatingSetting.ComboEditorOptions.ValueKey = "Key";
            updatingSetting.ComboEditorOptions.Virtualization = true;
            updatingSetting.ComboEditorOptions.DataSourceUrl = Url.Action("GetDistict_Generic", "Home");
            //updatingSetting.ComboEditorOptions.ItemTemplate = "{{if ${" + col4.Key + ".Text} == '' }}&nbsp;{{else}}${" + col4.Key + ".Text}{{/if}}";
            updating.ColumnSettings.Add(updatingSetting);


            ColumnUpdatingSetting updatingSetting2 = new ColumnUpdatingSetting();
            updatingSetting2.ColumnKey = "Name";
            updatingSetting2.EditorType = ColumnEditorType.Text;
            updatingSetting2.TextEditorOptions.MaxLength = 50;
            updating.ColumnSettings.Add(updatingSetting2);

            ColumnUpdatingSetting updatingSetting3 = new ColumnUpdatingSetting();
            updatingSetting3.EditorType = ColumnEditorType.Numeric;
            updatingSetting3.NumericEditorOptions.MinValue = 0;
            updating.ColumnSettings.Add(updatingSetting3);

            updating.RowEditDialogOptions.DialogTemplate = CreateDialogTemplateMarkup(updating, customColumns);

            return gridModel;
        }

        private string CreateDialogTemplateMarkup(GridUpdating updating, IList<CustomGridColumn> gridColumns)
        {
            IList<string> columnGroups = gridColumns.Select(x => x.ColumnGroup).Distinct().ToList();

            //The null Field Group should always get at the end
            if (columnGroups.Contains(null))
            {
                columnGroups.Remove(null);
                columnGroups.Add(null);
            }

            int groupCount = 1;

            string firstColumnMarkup = string.Empty;
            string secondColumnMarkup = string.Empty;

            string jqueryMarkup = "<div class='row-edit-dialog-container-cols' data-id='${Id}'>";
            foreach (string columnGroup in columnGroups)
            {
                bool isFirstColumn = groupCount <= columnGroups.Count / 2;

                var innerMarkup = "<table class='editdialogtable'><colgroup><col/><col/></colgroup><thead><tr><th colspan='2'>" + (columnGroup ?? string.Empty) + "</th></thead></tr><tbody>";
                foreach (CustomGridColumn field in gridColumns.Where(x => x.ColumnGroup == columnGroup))
                {
                    ColumnUpdatingSetting updateSetting = updating.ColumnSettings.FirstOrDefault(x => x.ColumnKey == field.Column.Key);

                    if (updateSetting == null || (updateSetting.ReadOnly.HasValue && updateSetting.ReadOnly.Value))
                        continue;

                    innerMarkup += "<tr><td class='editdialogheader'>" + field.Column.HeaderText + "</td><td class='editdialogfield' data-key='" + field.Column.Key + "' " + "><input data-editor-for-" + field.Column.Key + "='true' /></td></tr>";
                }

                innerMarkup += "</tbody></table>";

                if (isFirstColumn)
                    firstColumnMarkup += innerMarkup;
                else
                    secondColumnMarkup += innerMarkup;

                groupCount++;
            }

            //If we only have one group, then there is no need to have two separate columns, so don't add the additional divs in this case
            if (columnGroups.Count == 1)
                jqueryMarkup += firstColumnMarkup + secondColumnMarkup;
            else
                jqueryMarkup += "<div class='editdialogcol1'>" + firstColumnMarkup + "</div>" + "<div class='editdialogcol2'>" + secondColumnMarkup + "</div>";

            return jqueryMarkup;
        }

        //[GridDataSourceAction]
        //public ActionResult GetGridData()
        //{
        //    //return View(igGrid_MVC.Models.ProductsList.GetGridData());
        //    return View(pl.GetProductList());
        //}
    }
}